package org.apache.kudu.client;

import java.net.InetSocketAddress;
import java.util.Random;
import java.util.concurrent.Callable;
import java.util.concurrent.Executors;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.TimeUnit;

import javax.net.ssl.SSLException;

import org.jboss.netty.bootstrap.ClientBootstrap;
import org.jboss.netty.buffer.ChannelBuffers;
import org.jboss.netty.channel.Channel;
import org.jboss.netty.channel.ChannelFuture;
import org.jboss.netty.channel.ChannelHandlerContext;
import org.jboss.netty.channel.ChannelPipeline;
import org.jboss.netty.channel.ChannelPipelineFactory;
import org.jboss.netty.channel.ChannelStateEvent;
import org.jboss.netty.channel.Channels;
import org.jboss.netty.channel.ExceptionEvent;
import org.jboss.netty.channel.SimpleChannelUpstreamHandler;
import org.jboss.netty.channel.socket.nio.NioClientSocketChannelFactory;
import org.jboss.netty.handler.ssl.SslContext;
import org.jboss.netty.handler.ssl.SslHandler;
import org.jboss.netty.handler.ssl.util.InsecureTrustManagerFactory;

public class TestNettyStuff {

  static final boolean SSL = true;
  static final String HOST = System.getProperty("host", "ve0518");
  static final int PORT = Integer.parseInt(System.getProperty("port", "12034"));

  public static void main(String[] args) throws Exception {
      // Configure SSL.git
      final SslContext sslCtx;
      if (SSL) {
          sslCtx = SslContext.newClientContext(InsecureTrustManagerFactory.INSTANCE);
      } else {
          sslCtx = null;
      }

      // Configure the bootstrap.
      ClientBootstrap bootstrap = new ClientBootstrap(
              new NioClientSocketChannelFactory(
                      Executors.newCachedThreadPool(),
                      Executors.newCachedThreadPool()));

      try {
          bootstrap.setPipelineFactory(new ChannelPipelineFactory() {
              public ChannelPipeline getPipeline() {
                ChannelPipeline p = Channels.pipeline();
                p.addLast("echo", new EchoClientHandler());
                p.addLast("ssl", new SslHandler(sslCtx.newEngine()));
                return p;
              }
          });
          bootstrap.setOption("tcpNoDelay", true);
          ScheduledExecutorService exec = Executors.newScheduledThreadPool(1);

          Random r = new Random();
          while (true) {
            // Start the connection attempt.
            ChannelFuture future = bootstrap.connect(new InetSocketAddress(HOST, PORT));
            int delay = r.nextInt(20*1000);
            final Channel chan = future.getChannel();

            exec.schedule(new Callable<Void>() {
              @Override
              public Void call() throws Exception {
                chan.close();
                return null;
              }
            }, delay, TimeUnit.MICROSECONDS);

            // Wait until the connection is closed or the connection attempt fails.
            chan.getCloseFuture().sync();
            Thread.sleep(10);
          }
      } finally {
          // Shut down thread pools to exit.
          bootstrap.releaseExternalResources();
      }
  }

  public static class EchoClientHandler extends SimpleChannelUpstreamHandler {
    public EchoClientHandler() {
    }

    @Override
    public void channelConnected(final ChannelHandlerContext ctx,
                                 final ChannelStateEvent e) {
      Channel chan = ctx.getChannel();
      for (int i = 0; i < 1000; i++) {
        Channels.write(ctx.getChannel(), ChannelBuffers.copiedBuffer(
            "x\n".getBytes()));
      }
    }

    @Override
    public void exceptionCaught(ChannelHandlerContext ctx, ExceptionEvent e) {
      if (e.getCause() instanceof SSLException) {
        e.getCause().printStackTrace();
      }
    }
  }

}